#include "Stage.h"
#include "Stages.h"
#include "Main.h"
#include "Vram.h"
#include "Sprite.h"
#include "Chars.h"
#include "Status.h"
#include "CopyMemory.h"
#include "Man.h"
#include "Monster.h"
#include "Destruction.h"

extern void _deb();

ptr<Stage> pStage;
ptr<byte> pStageBytes;
byte[ColumnCount / ColumnsPerByte * FloorCount] CellMap;

constexpr byte CellByte_Space = 0;
constexpr byte CellByte_Item = (Char_Item - Char_Background) | (Color_Item << 4);
constexpr byte CellByte_CrackedFloor = (Char_CrackedFloor - Char_Background) | (Color_CrackedFloor << 4);
constexpr byte CellByte_HardFloor = (Char_HardFloor - Char_Background) | (Color_HardFloor << 4);
constexpr byte CellByte_Ladder_Full = (Char_Ladder_Full - Char_Background) | (Color_Ladder << 4);
constexpr byte CellByte_Ladder_Top = (Char_Ladder_Top - Char_Background) | (Color_Ladder << 4);

void InitStage() 
{
    {
        ptr<Stage> p;
        byte i, j;
        p = Stages;
        i = 0;
        j = 0;
        while (i < CurrentStage) {
            ++p;
            ++i;
            ++j;
            if (j >= StageCount) {
                p = Stages;
                j = 0;
            }
        }
        pStage = p;
        pStageBytes = pStage->bytes;
    }
}


static void DrawCellB(byte x, byte y, byte cb)
{
    byte c, color;
    if (cb != 0) {
        c = (cb & 0x0f) + Char_Background;
        color = cb >> 4;
    }
    else {
        c = 0;
        color = 0;
    }
    PutSprite(x, y, c, color);
}

static void DrawCell(byte x, byte y, byte b)
{
    static const byte[] UpperBytes = {
        CellByte_Space,
        CellByte_Space,
        CellByte_Ladder_Full,
        CellByte_Item,
    };
    static const byte[] LowerBytes = {
        CellByte_Space,
        CellByte_CrackedFloor,
        CellByte_Ladder_Full,
        CellByte_HardFloor,
    };
    byte upper, lower;
    upper = UpperBytes[(b >> 2) & 3];
    lower = LowerBytes[b & 3];
    if (upper == CellByte_Space && lower == CellByte_Ladder_Full) {
        upper = CellByte_Ladder_Top;
    }
    DrawCellB(x, y, upper);
    ++y;
    DrawCellB(x, y, lower);
}

void InitTrying()
{
    {
        byte i;
        StageTime = 25;
        i = pStage->itemCount;
        do {
            StageTime += 5;
            --i;
        } while (i != 0);
    }
    ItemCount = pStage->itemCount;
    HideAllSprites();
    ClearScreen();
    PrintStatus();

    CopyMemory(CellMap, pStageBytes, ColumnCount / ColumnsPerByte * FloorCount);
    {
        ptr<byte> pByte;
        byte y;
        pByte = pStageBytes;
        y = 0;
        repeat (FloorCount) {
            byte x;
            x = 0;
            repeat (ColumnCount / ColumnsPerByte) {
                byte b;
                b = *pByte; ++pByte;
                repeat (ColumnsPerByte) {
                    DrawCell(x, y, b);
                    ++x;
                    b >>= (8 / ColumnsPerByte);
                }
            }
            y += 2;
        }
    }
    InitMan();
    InitMonsters();
    InitDestructions();
    // UpdateSprites();
}


byte GetCell(byte column, byte floor)
{
    byte b;
    b = *CellMapPtr(column, floor);
    if ((column & 1) == 0) {
        return b & 0x0f;
    }
    else {
        return b >> 4;
    }
}


void SetCell(byte column, byte floor, byte cell)
{
    ptr<byte> pCell;
    byte mask;
    pCell = CellMapPtr(column, floor);
    if ((column & 1) == 0) {
        mask = 0xf0;
        cell &= 0x0f;
    }
    else {
        mask = 0x0f;
        cell <<= 4;
    }
    *pCell = (*pCell & mask) | cell;
}

// static void ShowAll()
// {
//     // UpdateSprites();
//     WaitTimer(4);
// }

// void ShowStage()
// {
//     byte x;
//     x = HScroll;
//     while (HScroll < 240) {
//         ++HScroll;
//         ShowAll();
//     }
//     while (HScroll > 208) {
//         --HScroll;
//         ShowAll();
//     }
//     while (HScroll < x) {
//         ++HScroll;
//         ShowAll();
//     }
// }